@@ -1,29 +1,35 @@
-## ---- include = FALSE---------------------------------------------------------
+## ----include = FALSE----------------------------------------------------------
 knitr::opts_chunk$set(
   collapse = TRUE,
   comment = "#>"
 )
 options(rmarkdown.html_vignette.check_title = FALSE)
 
-## ----setup--------------------------------------------------------------------
+## -----------------------------------------------------------------------------
+## Loading Required Libraries
 library(BGmisc)
 
-## -----------------------------------------------------------------------------
 set.seed(5)
-df_ped <- simulatePedigree(kpc = 4,
-                        Ngen = 4,
-                        sexR = .5,
-                        marR = .7)
+df_ped <- simulatePedigree(
+  kpc = 4,
+  Ngen = 4,
+  sexR = .5,
+  marR = .7
+)
 summary(df_ped)
 
 ## -----------------------------------------------------------------------------
 df_ped[21, ]
 
+## -----------------------------------------------------------------------------
+summarizeFamilies(df_ped, famID = "fam")$family_summary
+
 ## ----fig.width=8,fig.height=6-------------------------------------------------
 # Plot the simulated pedigree
 plotPedigree(df_ped)
 
 ## ----fig.width=8, fig.height=6------------------------------------------------
+set.seed(8)
 # Simulate a family with 3 generations
 df_ped_3 <- simulatePedigree(Ngen = 3)
 
